@@ -1,12 +1,14 @@
 package HTTP::Response;
-$HTTP::Response::VERSION = '6.12'; # TRIAL
+
 use strict;
 use warnings;
 
+our $VERSION = '6.19'; # TRIAL
+
 use base 'HTTP::Message';
 
 use HTTP::Status ();
-
+use Try::Tiny;
 
 sub new
 {
@@ -21,8 +23,9 @@ sub new
 sub parse
 {
     my($class, $str) = @_;
+    Carp::carp('Undefined argument to parse()') if $^W && ! defined $str;
     my $status_line;
-    if ($str =~ s/^(.*)\n//) {
+    if (defined $str && $str =~ s/^(.*)\n//) {
 	$status_line = $1;
     }
     else {
@@ -30,17 +33,21 @@ sub parse
 	$str = "";
     }
 
+    $status_line =~ s/\r\z// if defined $status_line;
+
     my $self = $class->SUPER::parse($str);
-    my($protocol, $code, $message);
-    if ($status_line =~ /^\d{3} /) {
-       # Looks like a response created by HTTP::Response->new
-       ($code, $message) = split(' ', $status_line, 2);
-    } else {
-       ($protocol, $code, $message) = split(' ', $status_line, 3);
+    if (defined $status_line) {
+        my($protocol, $code, $message);
+        if ($status_line =~ /^\d{3} /) {
+           # Looks like a response created by HTTP::Response->new
+           ($code, $message) = split(' ', $status_line, 2);
+        } else {
+           ($protocol, $code, $message) = split(' ', $status_line, 3);
+        }
+        $self->protocol($protocol) if $protocol;
+        $self->code($code) if defined($code);
+        $self->message($message) if defined($message);
     }
-    $self->protocol($protocol) if $protocol;
-    $self->code($code) if defined($code);
-    $self->message($message) if defined($message);
     $self;
 }
 
@@ -109,75 +116,100 @@ sub redirects {
     return reverse @r;
 }
 
-
-sub filename
-{
+sub filename {
     my $self = shift;
     my $file;
 
     my $cd = $self->header('Content-Disposition');
     if ($cd) {
-	require HTTP::Headers::Util;
-	if (my @cd = HTTP::Headers::Util::split_header_words($cd)) {
-	    my ($disposition, undef, %cd_param) = @{$cd[-1]};
-	    $file = $cd_param{filename};
+        require HTTP::Headers::Util;
+        if ( my @cd = HTTP::Headers::Util::split_header_words($cd) ) {
+            my ( $disposition, undef, %cd_param ) = @{ $cd[-1] };
 
-	    # RFC 2047 encoded?
-	    if ($file && $file =~ /^=\?(.+?)\?(.+?)\?(.+)\?=$/) {
-		my $charset = $1;
-		my $encoding = uc($2);
-		my $encfile = $3;
+            $file = $cd_param{filename};
 
-		if ($encoding eq 'Q' || $encoding eq 'B') {
-		    local($SIG{__DIE__});
-		    eval {
-			if ($encoding eq 'Q') {
-			    $encfile =~ s/_/ /g;
-			    require MIME::QuotedPrint;
-			    $encfile = MIME::QuotedPrint::decode($encfile);
-			}
-			else { # $encoding eq 'B'
-			    require MIME::Base64;
-			    $encfile = MIME::Base64::decode($encfile);
-			}
+            if (
+                exists $cd_param{'filename*'}
+                && ( my ( $fs_charset, $fs_lang, $fs_encfile )
+                    = $cd_param{'filename*'} =~ m/^([^']+)'(.*)'(.+)$/ )
+            ) {
+                # RFC 8187 encoded? Takes precedence.
+                warn
+                    q|Producers should use either UTF-8 or ISO-8859-1 character encoding.|
+                    unless $fs_charset =~ /^(utf-8|iso-8859-1)$/i;
 
-			require Encode;
-			require Encode::Locale;
-			Encode::from_to($encfile, $charset, "locale_fs");
-		    };
+                try {
+                    require URI::Escape;
+                    require Encode;
+                    require Encode::Locale;
+                    $fs_encfile = URI::Escape::uri_unescape($fs_encfile);
+                    Encode::from_to( $fs_encfile, $fs_charset, "locale_fs" );
 
-		    $file = $encfile unless $@;
-		}
-	    }
-	}
+                    $file = $fs_encfile;
+                }
+                catch {
+                    warn
+                        "Caught error while processing RFC 8187 'filename*' parameter: $_";
+                };
+            }
+            elsif (
+                $file
+                && ( my ( $charset, $encoding, $encfile )
+                    = $file =~ /^=\?(.+?)\?(.+?)\?(.+)\?=$/ )
+            ) {
+                # RFC 2047 encoded?
+                if ( $encoding eq 'Q' || $encoding eq 'B' ) {
+                    try {
+                        if ( $encoding eq 'Q' ) {
+                            $encfile =~ s/_/ /g;
+                            require MIME::QuotedPrint;
+                            $encfile = MIME::QuotedPrint::decode($encfile);
+                        }
+                        else {    # $encoding eq 'B'
+                            require MIME::Base64;
+                            $encfile = MIME::Base64::decode($encfile);
+                        }
+
+                        require Encode;
+                        require Encode::Locale;
+                        Encode::from_to( $encfile, $charset, "locale_fs" );
+
+                        $file = $encfile;
+                    }
+                    catch {
+                        warn
+                            "Caught error while processing RFC 2047 'filename' parameter: $_";
+                    };
+                }
+            }
+        }
     }
 
-    unless (defined($file) && length($file)) {
-	my $uri;
-	if (my $cl = $self->header('Content-Location')) {
-	    $uri = URI->new($cl);
-	}
-	elsif (my $request = $self->request) {
-	    $uri = $request->uri;
-	}
+    unless ( defined($file) && length($file) ) {
+        my $uri;
+        if ( my $cl = $self->header('Content-Location') ) {
+            $uri = URI->new($cl);
+        }
+        elsif ( my $request = $self->request ) {
+            $uri = $request->uri;
+        }
 
-	if ($uri) {
-	    $file = ($uri->path_segments)[-1];
-	}
+        if ($uri) {
+            $file = ( $uri->path_segments )[-1];
+        }
     }
 
     if ($file) {
-	$file =~ s,.*[\\/],,;  # basename
+        $file =~ s,.*[\\/],,;    # basename
     }
 
-    if ($file && !length($file)) {
-	$file = undef;
+    if ( $file && !length($file) ) {
+        $file = undef;
     }
 
     $file;
 }
 
-
 sub as_string
 {
     my $self = shift;
@@ -344,7 +376,7 @@ HTTP::Response - HTTP style response message
 
 =head1 VERSION
 
-version 6.12
+version 6.19
 
 =head1 SYNOPSIS
 
@@ -490,10 +522,10 @@ received some redirect responses first.
 
 If none of these sources provide an absolute URI, undef is returned.
 
-When the LWP protocol modules produce the HTTP::Response object, then
-any base URI embedded in the document (step 1) will already have
-initialized the "Content-Base:" header. This means that this method
-only performs the last 2 steps (the content is not always available
+When the LWP protocol modules produce the HTTP::Response object, then any base
+URI embedded in the document (step 1) will already have initialized the
+"Content-Base:" header. (See L<LWP::UserAgent/parse_head>).  This means that
+this method only performs the last 2 steps (the content is not always available
 either).
 
 =item $r->filename
@@ -512,9 +544,20 @@ order):
 
 =item 1.
 
-A "Content-Disposition:" header in the response.  Proper decoding of
-RFC 2047 encoded filenames requires the C<MIME::QuotedPrint> (for "Q"
-encoding), C<MIME::Base64> (for "B" encoding), and C<Encode> modules.
+A "Content-Disposition:" header in the response.
+
+This uses the C<filename> or the C<filename*> parameter. If both are
+present, C<filename*> is preferred.
+
+Proper decoding of RFC 2047 encoded filenames in the C<filename>
+parameter requires the C<MIME::QuotedPrint> (for "Q" encoding),
+C<MIME::Base64> (for "B" encoding), and C<Encode> modules.
+
+Proper decoding of RFC 8187 encoded filenames in the C<filename*>
+parameter requires the L<URI::Escape> and L<Encode> modules.
+
+If the content needs to be decoded, it will be returned as a string encoded
+using the C<locale_fs> encoding as defined in the L<Encode::Locale> module.
 
 =item 2.
 
@@ -644,7 +687,7 @@ Gisle Aas <gisle@activestate.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 1994-2016 by Gisle Aas.
+This software is copyright (c) 1994-2017 by Gisle Aas.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
